@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2021  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
+ * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * As an exception, when this program is distributed through (i) the App Store
  * by Apple Inc.; (ii) the Mac App Store by Apple Inc.; or (iii) Google Play
@@ -31,6 +31,8 @@
 
 namespace libnormaliz {
 
+using std::string;
+
 namespace Type {
 enum InputType {
     //
@@ -44,6 +46,7 @@ enum InputType {
     lattice,
     saturation,
     rational_lattice,
+    monoid,
     //
     // inhomogeneous generators
     //
@@ -68,15 +71,20 @@ enum InputType {
     inhom_congruences,
     inhom_excluded_faces,
     //
-    // linearforms
+    // linear forms
     //
     grading,
     dehomogenization,
     //
+    // lattice ideals and friends
+    //
+    lattice_ideal,
+    toric_ideal,
+    normal_toric_ideal,
+    //
     // special
     open_facets,
     projection_coordinates,
-    lattice_ideal,
     //
     // precomputed data
     //
@@ -101,7 +109,7 @@ enum InputType {
     add_inhom_inequalities,
     add_equations,
     add_inhom_equations
-    
+
 };
 }  // end namespace Type
 
@@ -109,7 +117,7 @@ using Type::InputType;
 
 /* converts a string to an InputType
  * throws an BadInputException if the string cannot be converted */
-InputType to_type(const std::string& type_string);
+InputType to_type(const string& type_string);
 
 /* gives the difference of the number of columns to the dimension */
 long type_nr_columns_correction(InputType type);
@@ -128,15 +136,30 @@ enum Param {
     autom_codim_bound_vectors,
     block_size_hollow_tri,
     decimal_digits,
+    gb_degree_bound,
+    gb_min_degree,
     not_a_num_param
 };
 }  // end namespace NumParam
 
-bool isNumParam(NumParam::Param& numpar, const std::string& type_string);
-NumParam::Param to_numpar(const std::string& type_string);
-std::string numpar_to_string(const NumParam::Param& numpar);
+bool isNumParam(NumParam::Param& numpar, const string& type_string);
+NumParam::Param to_numpar(const string& type_string);
+string numpar_to_string(const NumParam::Param& numpar);
 
-inline InputType to_type(const std::string& type_string) {
+namespace PolyParam {
+enum Param {
+    polynomial,
+    polynomial_equations,
+    polynomial_inequalities,
+    not_a_poly_param
+};
+}  // end namespace PolyParam
+
+bool isPolyParam(PolyParam::Param& polypar, const string& type_string);
+PolyParam::Param to_polypar(const string& type_string);
+string polypar_to_string(const PolyParam::Param& polypar);
+
+inline InputType to_type(const string& type_string) {
     if (type_string == "0" || type_string == "1" || type_string == "2" || type_string == "3" || type_string == "4" ||
         type_string == "5" || type_string == "6" || type_string == "hyperplanes" || type_string == "10") {
         throw BadInputException("Error: deprecated type \"" + type_string + "\", please use new type string!");
@@ -187,9 +210,15 @@ inline InputType to_type(const std::string& type_string) {
     if (type_string == "signs") {
         return Type::signs;
     }
-    if (type_string == "10" || type_string == "lattice_ideal") {
+    if (type_string == "lattice_ideal") {
         return Type::lattice_ideal;
     }
+    if ( type_string == "toric_ideal") {
+        return Type::toric_ideal;
+    }
+    if ( type_string == "normal_toric_ideal") {
+        return Type::normal_toric_ideal;
+    }
     if (type_string == "grading") {
         return Type::grading;
     }
@@ -208,6 +237,9 @@ inline InputType to_type(const std::string& type_string) {
     if (type_string == "saturation") {
         return Type::saturation;
     }
+    if (type_string == "monoid") {
+        return Type::monoid;
+    }
     if (type_string == "cone") {
         return Type::cone;
     }
@@ -243,16 +275,15 @@ inline InputType to_type(const std::string& type_string) {
     if (type_string == "extreme_rays") {
         return Type::extreme_rays;
     }
-    
+
     if (type_string == "maximal_subspace") {
         return Type::maximal_subspace;
     }
-    
+
     if (type_string == "generated_lattice") {
         return Type::generated_lattice;
     }
 
-
     if (type_string == "scale") {
         return Type::scale;
     }
@@ -260,7 +291,7 @@ inline InputType to_type(const std::string& type_string) {
     if (type_string == "add_cone") {
         return Type::add_cone;
     }
-    
+
     if (type_string == "add_subspace") {
         return Type::add_subspace;
     }
@@ -272,7 +303,7 @@ inline InputType to_type(const std::string& type_string) {
     if (type_string == "add_inequalities") {
         return Type::add_inequalities;
     }
-    
+
     if (type_string == "add_equations") {
         return Type::add_equations;
     }
@@ -280,7 +311,7 @@ inline InputType to_type(const std::string& type_string) {
     if (type_string == "add_inhom_inequalities") {
         return Type::add_inhom_inequalities;
     }
-    
+
     if (type_string == "add_inhom_equations") {
         return Type::add_inhom_equations;
     }
@@ -294,7 +325,7 @@ inline long type_nr_columns_correction(InputType t) {
         return -1;
     if (t == Type::congruences || t == Type::vertices || t == Type::polyhedron || t == Type::inhom_inequalities ||
         t == Type::inhom_equations || t == Type::hilbert_basis_rec_cone || t == Type::add_inhom_inequalities ||
-        t == Type::add_vertices|| t == Type::add_inhom_equations || t == Type::inhom_excluded_faces)
+        t == Type::add_vertices || t == Type::add_inhom_equations || t == Type::inhom_excluded_faces)
         return 1;
     if (t == Type::inhom_congruences)
         return 2;
@@ -311,7 +342,29 @@ inline bool type_is_vector(InputType type) {
     return false;
 }
 
-inline  NumParam::Param to_numpar(const std::string& type_string) {
+inline bool is_inequalities(InputType type) {
+    if (type == Type::signs || type == Type::strict_signs || type == Type::inequalities || type ==
+        Type::strict_inequalities || type == Type::inhom_inequalities || type == Type::excluded_faces
+           || type == Type::inhom_excluded_faces) {
+        return true;
+    }
+    return false;
+}
+
+// generators definiong something convex
+inline bool is_generators(InputType type) {
+    if (type == Type::cone || type == Type::cone_and_lattice
+           || type == Type::polyhedron  || type == Type::vertices
+           || type == Type::polytope || type == Type::rees_algebra
+           || type == Type::monoid || type == Type::integral_closure
+           || type == Type::normalization) {
+        return true;
+    }
+    return false;
+}
+
+
+inline NumParam::Param to_numpar(const string& type_string) {
     if (type_string == "expansion_degree")
         return NumParam::expansion_degree;
     if (type_string == "nr_coeff_quasipol")
@@ -324,11 +377,15 @@ inline  NumParam::Param to_numpar(const std::string& type_string) {
         return NumParam::block_size_hollow_tri;
     if (type_string == "decimal_digits")
         return NumParam::decimal_digits;
+    if (type_string == "gb_degree_bound")
+        return NumParam::gb_degree_bound;
+    if (type_string == "gb_min_degree")
+        return NumParam::gb_min_degree;
 
     return NumParam::not_a_num_param;
 }
 
-inline  std::string numpar_to_string(const NumParam::Param& numpar) {
+inline string numpar_to_string(const NumParam::Param& numpar) {
     if (numpar == NumParam::expansion_degree)
         return "expansion_degree";
     if (numpar == NumParam::nr_coeff_quasipol)
@@ -343,19 +400,55 @@ inline  std::string numpar_to_string(const NumParam::Param& numpar) {
         return "block_size_hollow_tri";
     if (numpar == NumParam::decimal_digits)
         return "decimal_digits";
+    if (numpar == NumParam::gb_degree_bound)
+        return "gb_degree_bound";
+    if (numpar == NumParam::gb_min_degree)
+        return "gb_min_degree";
     if (numpar == NumParam::not_a_num_param)
         return "not_a_num_param";
     assert(false);
-    return std::string(); // silence compiler warning
+    return string();  // silence compiler warning
 }
 
-inline  bool isNumParam(NumParam::Param& numpar, const std::string& type_string) {
+inline bool isNumParam(NumParam::Param& numpar, const string& type_string) {
     numpar = to_numpar(type_string);
     if (numpar == NumParam::not_a_num_param)
         return false;
     return true;
 }
 
+inline PolyParam::Param to_polypar(const string& type_string) {
+    if (type_string == "polynomial")
+        return PolyParam::polynomial;
+    if (type_string == "polynomial_equations")
+        return PolyParam::polynomial_equations;
+    if (type_string == "polynomial_inequalities")
+        return  PolyParam::polynomial_inequalities;
+
+    return PolyParam::not_a_poly_param;
+}
+
+inline string polypar_to_string(const PolyParam::Param& polypar) {
+    if (polypar == PolyParam::polynomial)
+        return "polynomial";
+    if (polypar == PolyParam::polynomial_equations)
+        return "polynomial_equations";
+    if (polypar == PolyParam::polynomial_inequalities)
+        return "polynomial_inequalities";
+    if (polypar == PolyParam::not_a_poly_param)
+        return "not_a_poly_param";
+    assert(false);
+    return string();  // silence compiler warning
+}
+
+inline bool isPolyParam(PolyParam::Param& polypar, const string& type_string) {
+    polypar = to_polypar(type_string);
+    if (polypar == PolyParam::not_a_poly_param)
+        return false;
+    return true;
+}
+
+
 } /* end namespace libnormaliz */
 
 #endif /* LIBNORMALIZ_H_ */
